local Tasks = Tasks
local L = Tasks.L
local TasksUtils = TasksUtils
local db
local itemFramePool, headerFramePool, frameLookup = {}, {}, {}
local taskFrameCount, headerFrameCount = 1,1

Tasks:RegisterCallback("Tasks-Initialize", "DisplayInit", 1)

function Tasks:DisplayInit()
  db = Tasks.db
  self:CreateTasksFrame()
  
  Tasks:RegisterCallback("Tasks-TaskCompleted", "DisplayTaskCompleted")
  Tasks:RegisterCallback("Tasks-TaskReset", "DisplayTaskReset")
  Tasks:RegisterCallback("Tasks-EnabledChanged", "DisplayEnabledChanged")
  Tasks:RegisterCallback("Tasks-HideCompletedChanged", "DisplayHideCompletedChanged")
end

-- self.TasksFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
--                                           --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
--                                           tile = true, tileSize = 16, edgeSize = 16, 
--                                           insets = { left = 4, right = 4, top = 4, bottom = 4 }});

function Tasks:CreateTasksFrame()
  local frameSettings = db.global.displayOptions.frameSettings
  self.TasksFrame = CreateFrame("Button","TasksFrame",UIParent, "TasksTemplate")
  if not frameSettings.locked then Tasks.TasksFrame:RegisterForDrag("LeftButton")
  else Tasks.TasksFrame:RegisterForDrag() end
  self.TasksFrame:RegisterForDrag("LeftButton")
  self.TasksFrame:SetPushedTextOffset(0,0)

  self.TasksFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            });

  self.TasksFrame:SetBackdropColor(0,0,255,0);
  
  self.TasksFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            });

  self.TasksFrame:SetBackdropColor(255,0,255,0);
  --set a default location. if user moves it, blizzard will save to layout-local.txt and restore automatically
  self.TasksFrame:SetPoint("CENTER", UIParent, "CENTER", 300, 380)

  self:CreateTaskListFrames(self.TasksFrame)
end

function Tasks:NewItemFrame(taskId, task, listFrame)
  local tempFrame
  if table.getn(itemFramePool) > 0 then
    tempFrame = itemFramePool[1]
    table.remove(itemFramePool,1)
  else
    tempFrame = CreateFrame("Frame","TasksItemFrame"..taskFrameCount,listFrame.childrenFrame, "TasksItemTemplate")
    tempFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            })
    
    tempFrame:SetBackdropColor(255,255,0,0)
    tempFrame:SetAlpha(1.0)
    taskFrameCount = taskFrameCount+1
  end
    tempFrame.task = task
  frameLookup[taskId] = tempFrame
  return tempFrame
end

function Tasks:NewHeaderFrame(listId,list)
  local tempFrame
  if table.getn(headerFramePool) > 0 then
    tempFrame = headerFramePool[1]
    table.remove(headerFramePool,1)
  else
    tempFrame = CreateFrame("Frame","TasksHeaderFrame"..headerFrameCount,self.TasksFrame, "TasksHeaderTemplate")
    tempFrame.childrenFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            })
    
    tempFrame.childrenFrame:SetBackdropColor(255,0,0,0)
    tempFrame:SetAlpha(1.0)
    headerFrameCount = headerFrameCount+1

    tempFrame.childrenFrame.origSetHeight = tempFrame.childrenFrame.SetHeight
    function tempFrame.childrenFrame:SetHeight(newHeight)
      self:origSetHeight(newHeight)
      self:GetParent():SetHeight(newHeight+15)
    end
  end
  tempFrame.children = {}
  tempFrame.list = list
  frameLookup[listId] = tempFrame
  return tempFrame
end

function Tasks:CreateTaskListFrames(anchorFrame)
  local headerTemp
  for listId, list in Tasks:OrderedTaskLists(db.global.lists) do
    local listHeaderFrame = Tasks:NewHeaderFrame(listId, list)
    listHeaderFrame:SetPoint("TOPLEFT", headerTemp or anchorFrame, "BOTTOMLEFT", 0, 0)
    listHeaderFrame:Show()
    
    if headerTemp then headerTemp.nextHeaderFrame = listHeaderFrame end
    listHeaderFrame.prevHeaderFrame = headerTemp
    
    anchorFrame = listHeaderFrame.childrenFrame
    headerTemp = listHeaderFrame
    
    --create tasks
    local taskTemp
    for taskId, task in Tasks:OrderedTasks(list.tasks) do
      local taskFrame = Tasks:NewItemFrame(taskId, task, listHeaderFrame)
      taskFrame:SetPoint("TOPLEFT", anchorFrame, listHeaderFrame.firstTask and "BOTTOMLEFT" or "TOPLEFT", 0, 0)
      taskFrame:Show()
      listHeaderFrame.firstTask = listHeaderFrame.firstTask or taskFrame
      listHeaderFrame.lastTask = taskFrame
      table.insert(listHeaderFrame.children,taskFrame)
      if taskTemp then taskTemp.nextTaskFrame = taskFrame end
      taskFrame.prevTaskFrame = taskTemp
      
      taskFrame.listFrame = listHeaderFrame
      anchorFrame = taskFrame
      taskTemp = taskFrame
      listHeaderFrame.childrenFrame:SetHeight(listHeaderFrame.childrenFrame:GetHeight()+15)
    end
    
    if not list.displayOptions.expanded then
      listHeaderFrame.childrenFrame:SetHeight(0)
      listHeaderFrame.childrenFrame:Hide()
    end
  end
end


function Tasks:DisplayTaskCompleted(task)
  frameLookup[task.id].checkbox:SetChecked(true)
    
  --check other tasks complete also to mark off list
  local list = db.global.lists[task.listId]
  local allDone = true;
  for _,v in pairs(list.tasks) do
    if v ~= task and not v.completedChars[self.currentChar].completed then
      allDone = false
      break;
    end
  end
  if allDone then
    frameLookup[task.listId].checkbox:SetChecked(true)
  end
  
  --hide tasks if option enabled
  if(db.global.displayOptions.hideCompletedTasks) then
  print("hiding")
    if not allDone then
      --only one task to remove
      Tasks:HideTask(task)
    else
      --entire list done
      Tasks:HideList(list)
    end
    --listHeaderFrame.childrenFrame:SetHeight(listHeaderFrame.childrenFrame:GetHeight()+15)
  end
end

function Tasks:HideTask(task)
  print("hiding task")
  local taskFrame = frameLookup[task.id]
  
  --correct frame pointers
  if (taskFrame.listFrame.firstTask == taskFrame) then
    taskFrame.listFrame.firstTask = taskFrame.nextTaskFrame
  end
  if taskFrame.prevTaskFrame then 
    taskFrame.prevTaskFrame.nextTaskFrame = taskFrame.nextTaskFrame
  end
  if taskFrame.nextTaskFrame then
    taskFrame.nextTaskFrame.prevTaskFrame = taskFrame.prevTaskFrame
    taskFrame.nextTaskFrame:ClearAllPoints()
    taskFrame.nextTaskFrame:SetPoint("TOPLEFT", taskFrame.prevTaskFrame or taskFrame.listFrame.childrenFrame, taskFrame.prevTaskFrame and "BOTTOMLEFT" or "TOPLEFT", 0, 0)
  end
  if (taskFrame.listFrame.lastTask == taskFrame) then
    taskFrame.listFrame.lastTask = taskFrame.prevTaskFrame
  end
  taskFrame:Hide()
  taskFrame.listFrame.childrenFrame:SetHeight(taskFrame.listFrame.childrenFrame:GetHeight()-15)
end

function Tasks:HideList(list)
  print("do stuff")
end

function Tasks:ShowTask(task)

end

function Tasks:ShowList(task)

end


function Tasks:DisplayTaskReset(task)
  frameLookup[task.id].checkbox:SetChecked(false)
  frameLookup[task.listId].checkbox:SetChecked(false)
end

function Tasks:DisplayEnabledChanged(enabled)
  if(enabled) then
    Tasks.taskFrame.Show()
  else
    Tasks.taskFrame.Hide()
  end
end

function Tasks:DisplayHideCompletedChanged(enabled)

end

function Tasks:TaskListClicked(checkbox)
  for k,v in ipairs(checkbox:GetParent().children) do
    if checkbox:GetChecked() then
      v.task.completedChars[self.currentChar].completed = checkbox:GetChecked()
      v.task.completedChars[self.currentChar].timestamp = time()
      v.checkbox:SetChecked(true)
    else
      v.task.completedChars[self.currentChar] = nil
      v.checkbox:SetChecked(false)
    end 
  end
end

function Tasks:TaskHeaderClicked(button)
local list = button:GetParent().list
  if list.displayOptions.expanded then
    --hide the list
    list.displayOptions.expanded = false
    _G[button:GetParent():GetName() .. "_CheckButtonText"]:SetText("|cFF76DEAF + "..list.name.."|r")
    button:GetParent().childrenFrame:SetHeight(0)
    button:GetParent().childrenFrame:Hide()
  else
    --show the list
    list.displayOptions.expanded = true
    _G[button:GetParent():GetName() .. "_CheckButtonText"]:SetText("|cFF76DEAF - "..list.name.."|r")
    button:GetParent().childrenFrame:SetHeight(table.getn(button:GetParent().children)*15)
    button:GetParent().childrenFrame:Show()
  end
end

local function __genOrderedIndexTaskList( t )
    local orderedIndex = {}
    for key in pairs(t) do
        table.insert( orderedIndex, key )
    end
    table.sort( orderedIndex, function(a,b)
                                return t[a].order < t[b].order
                              end)
    return orderedIndex
end

local function orderedNextTaskList(t, state)
    if state == nil then
        -- the first time, generate the index
        t.__orderedIndex = __genOrderedIndexTaskList( t )
        key = t.__orderedIndex[1]
        return key, t[key]
    end
    -- fetch the next value
    key = nil
    for i = 1,table.getn(t.__orderedIndex) do
        if t.__orderedIndex[i] == state then
            key = t.__orderedIndex[i+1]
        end
    end

    if key then
        return key, t[key]
    end
    
    t.__orderedIndex = nil
    return
end

function Tasks:OrderedTaskLists(t)
    return orderedNextTaskList, t, nil
end

local function __genOrderedIndexTask( t )
  local orderedIndex = {}
  for key in pairs(t) do
    if(not db.global.displayOptions.hideCompletedTasks or not t[key].completedChars or not t[key].completedChars[Tasks.currentChar].completed) then
      table.insert( orderedIndex, key )
    end
  end
  table.sort( orderedIndex, function(a,b)
                              return t[a].order < t[b].order
                            end)
  return orderedIndex
end

local function orderedNextTask(t, state)
    if state == nil then
        -- the first time, generate the index
        t.__orderedIndex = __genOrderedIndexTask( t )
        key = t.__orderedIndex[1]
        return key, t[key]
    end
    -- fetch the next value
    key = nil
    for i = 1,table.getn(t.__orderedIndex) do
        if t.__orderedIndex[i] == state then
            key = t.__orderedIndex[i+1]
        end
    end

    if key then
        return key, t[key]
    end
    
    t.__orderedIndex = nil
    return
end

function Tasks:OrderedTasks(t)
    return orderedNextTask, t, nil
end