Tasks = LibStub("AceAddon-3.0"):NewAddon("Tasks", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
Tasks.L = LibStub("AceLocale-3.0"):GetLocale("Tasks", false)
TasksUtils = {}
Tasks.callbacks = setmetatable({}, {__index = function(tbl, key) tbl[key] = {} return tbl[key] end})


function Tasks:OnInitialize()
  self.db = LibStub("AceDB-3.0"):New("TasksDB", self.defaults["db"], true)
  self:UpgradeDBVersion()
  
  --retrieve and store character info
  local char = UnitName("player") .. " - " .. GetRealmName()
  -- local char =  GetUnitName("player", true")
  self.currentChar = char
  self.db.global.characters[char].name = UnitName("player")
  self.db.global.characters[char].realm = GetRealmName()
  self.db.global.characters[char].class = select(2,UnitClass("player"))
  self.db.global.characters[char].level = UnitLevel("player")
  self.db.global.characters[char].race = select(2, UnitRace("player"))
  self.db.global.characters[char].faction = UnitFactionGroup("player")
  
  --initialize various modules
  self:Fire("Tasks-Initialize",nil)
end

function Tasks:OnEnable()
  self:CheckReset()
end

function Tasks:OnDisable()

end

function Tasks:UpgradeDBVersion()
  if not(self.db.global.dbVersion) then
    self.db.global.dbVersion = self.constants.CURRENT_DB_VERSION
  end
  
  if(self.db.global.dbVersion == 1) then
    self.db.global.encounters = nil
    self.db.global.dbVersion = 2
  end
  
  if(self.db.global.dbVersion == 2) then
    for k,v in pairs(self.db.global.lists) do
      v.id = k
      for k2,v2 in pairs(v.tasks) do
        v2.id = k2
        v2.listId = k
      end
    end
    self.db.global.editTask = nil
    self.db.global.dbVersion = 3
  end
  
  if(self.db.global.dbVersion == 3) then
    local listCount = 0
    for k,v in pairs(self.db.global.lists) do
      listCount = listCount + 1
      v.order = listCount
    end
    self.db.global.listCount = listCount
    self.db.global.dbVersion = 3
  end
end

function Tasks:Fire(event, ...)
  for k in Tasks:OrderedTaskCallbacks(Tasks.callbacks[event]) do
    loadstring('Tasks:'..k..'(...)')(...)
  end
end

function Tasks:RegisterCallback(event, method, priority)
  if not priority then priority = 0 end
  Tasks.callbacks[event][method] = priority
end

function Tasks:UnregisterCallback(event, method)
  Tasks.callbacks[event][method]= nil
end

local function __genOrderedIndexTaskCallback( t )
    local orderedIndex = {}
    for key in pairs(t) do
        table.insert( orderedIndex, key )
    end
    table.sort( orderedIndex, function(a,b)
                                return t[a] > t[b]
                              end)
    return orderedIndex
end

local function orderedNextTaskCallback(t, state)
    if state == nil then
        -- the first time, generate the index
        t.__orderedIndex = __genOrderedIndexTaskCallback( t )
        key = t.__orderedIndex[1]
        return key, t[key]
    end
    -- fetch the next value
    key = nil
    for i = 1,table.getn(t.__orderedIndex) do
        if t.__orderedIndex[i] == state then
            key = t.__orderedIndex[i+1]
        end
    end

    if key then
        return key, t[key]
    end
    
    t.__orderedIndex = nil
    return
end

function Tasks:OrderedTaskCallbacks(t)
    return orderedNextTaskCallback, t, nil
end